import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np


class _Chromosome(object):

    def __init__(self,
                 chr_len_db: dict,
                 chr_order: list = None,
                 bed_data: list = None,
                 centro_db: dict = None,
                 value_type: str = "numeric",
                 orientation: str = "vertical",
                 cmap: str = "gist_rainbow",
                 cmap_parts: int = 100,
                 s: any = None):

        self.__chr_len_db = chr_len_db
        self.__chr_order = chr_order
        self.__bed_data = bed_data
        self.__centro_db = centro_db
        self.__value_type = value_type.lower()
        self.__avail_types = {'numeric', 'color', 'marker'}
        if self.__value_type not in self.__avail_types:
            raise ValueError("value_type must in %s" % ','.join(list(self.__avail_types)))

        self.__s = s
        self.__cmap = plt.get_cmap(cmap)
        if cmap_parts <= 0:
            raise ValueError("cmap_parts must larger than 0")

        self.__cmap_parts = cmap_parts
        self.__orientation = orientation

    # pos: 0~3, means top_right, top_left, buttom_left, buttom_right
    @staticmethod
    def __generate_arc(r, c, pos, ratio):
        x = []
        y = []
        theta = pos * np.pi / 2.0
        while theta <= (pos + 1) * np.pi / 2.0:
            x.append(r * np.cos(theta) + c[0])
            y.append(r * np.sin(theta) * ratio + c[1])
            theta += 0.01
        return x, y

    def plot(self, ax, fig, kwargs):
        if self.__orientation != "vertical" and self.__orientation != "horizontal":
            print("Warning: orientation must be \"vertical\" or \"horizontal\", reset to \"vertical\"")
            self.__orientation = "vertical"
        for i in ax.spines:
            ax.spines[i].set_visible(False)
        ax.tick_params('both', length=0)
        chr_list = sorted(self.__chr_len_db)
        chr_cnt = len(chr_list)
        max_height = 0
        for _ in self.__chr_len_db:
            if max_height < self.__chr_len_db[_]:
                max_height = self.__chr_len_db[_]

        # Plot chromosomes
        fig_w, fig_h = fig.get_size_inches() * fig.dpi
        if self.__orientation == 'horizontal':
            ratio = max_height * 1. / chr_cnt * (fig_h * 1. / fig_w)
        else:
            ratio = max_height * 1. / chr_cnt * (fig_w * 1. / fig_h)
        for i in range(chr_cnt):
            if self.__chr_order:
                chrn = self.__chr_order[i]
            else:
                chrn = chr_list[i]
            x = i
            r = 0.35

            height = self.__chr_len_db[chrn]

            # Draw two end arc
            for j in [2, 3]:
                arc_x, arc_y = self.__generate_arc(r, [x, r * ratio], j, ratio)
                if self.__orientation == "horizontal":
                    arc_x, arc_y = arc_y, arc_x
                plt.plot(arc_x, arc_y, **kwargs)
            for j in [0, 1]:
                arc_x, arc_y = self.__generate_arc(r, [x, height - r * ratio], j, ratio)
                if self.__orientation == "horizontal":
                    arc_x, arc_y = arc_y, arc_x
                plt.plot(arc_x, arc_y, **kwargs)

            if self.__centro_db and chrn in self.__centro_db:
                # if centromere found, draw it
                x_start = x - 0.35
                x_end = x - 0.35
                y_start = r * ratio
                y_end = self.__centro_db[chrn] - r * ratio
                if self.__orientation == "horizontal":
                    x_start, y_start = y_start, x_start
                    x_end, y_end = y_end, x_end
                plt.plot([x_start, x_end], [y_start, y_end], **kwargs)

                x_start = x - 0.35
                x_end = x - 0.35
                y_start = self.__centro_db[chrn] + r * ratio
                y_end = height - r * ratio
                if self.__orientation == "horizontal":
                    x_start, y_start = y_start, x_start
                    x_end, y_end = y_end, x_end
                plt.plot([x_start, x_end], [y_start, y_end], **kwargs)

                x_start = x + 0.35
                x_end = x + 0.35
                y_start = r * ratio
                y_end = self.__centro_db[chrn] - r * ratio
                if self.__orientation == "horizontal":
                    x_start, y_start = y_start, x_start
                    x_end, y_end = y_end, x_end
                plt.plot([x_start, x_end], [y_start, y_end], **kwargs)

                x_start = x + 0.35
                x_end = x + 0.35
                y_start = self.__centro_db[chrn] + r * ratio
                y_end = height - r * ratio
                if self.__orientation == "horizontal":
                    x_start, y_start = y_start, x_start
                    x_end, y_end = y_end, x_end
                plt.plot([x_start, x_end], [y_start, y_end], **kwargs)

                for j in [2, 3]:
                    arc_x, arc_y = self.__generate_arc(r, [x, self.__centro_db[chrn] + r * ratio], j, ratio)
                    if self.__orientation == "horizontal":
                        arc_x, arc_y = arc_y, arc_x
                    plt.plot(arc_x, arc_y, **kwargs)
                for j in [0, 1]:
                    arc_x, arc_y = self.__generate_arc(r, [x, self.__centro_db[chrn] - r * ratio], j, ratio)
                    if self.__orientation == "horizontal":
                        arc_x, arc_y = arc_y, arc_x
                    plt.plot(arc_x, arc_y, **kwargs)
            else:
                x_start = x - 0.35
                x_end = x - 0.35
                y_start = r * ratio
                y_end = height - r * ratio
                if self.__orientation == "horizontal":
                    x_start, y_start = y_start, x_start
                    x_end, y_end = y_end, x_end
                plt.plot([x_start, x_end], [y_start, y_end], **kwargs)

                x_start = x + 0.35
                x_end = x + 0.35
                y_start = r * ratio
                y_end = height - r * ratio
                if self.__orientation == "horizontal":
                    x_start, y_start = y_start, x_start
                    x_end, y_end = y_end, x_end
                plt.plot([x_start, x_end], [y_start, y_end], **kwargs)

        # set default color
        if not kwargs.get("color") and not kwargs.get("c"):
            for line in ax.get_lines():
                line.set_color("black")

        xticks = []
        for i in range(chr_cnt):
            xticks.append(i)
        if self.__chr_order:
            xlabels = self.__chr_order
        else:
            xlabels = chr_list

        yticks = []
        ylabels = []
        # add label each Mb
        for pos in range(0, int(max_height), int(1e6)):
            yticks.append(pos)
            ylabels.append("%.0fMb" % (pos / 1e6))
        next_pos = round(max_height / 1e6) * 1e6
        if next_pos > yticks[-1]:
            yticks.append(next_pos)
            ylabels.append("%.0fMb" % (next_pos / 1e6))
        if self.__orientation == 'horizontal':
            xticks, yticks = yticks, xticks
            xlabels, ylabels = ylabels, xlabels
        ax.set_xticks(xticks, xlabels, rotation=90 if self.__orientation == "vertical" else 0)
        ax.set_yticks(yticks, ylabels)

        clb = None

        if not self.__bed_data:
            return None
        if self.__chr_order:
            chr_idx_db = {self.__chr_order[_]: _ for _ in range(chr_cnt)}
        else:
            chr_idx_db = {chr_list[_]: _ for _ in range(chr_cnt)}

        if self.__value_type == 'numeric':
            # Init colormap
            max_val = None
            min_val = None
            for _, _, _, val in self.__bed_data:
                if not max_val or val > max_val:
                    max_val = val
                if not min_val or val < min_val:
                    min_val = val
            norm = mpl.colors.Normalize(vmin=min_val, vmax=max_val, clip=True)
            mapper = mpl.cm.ScalarMappable(norm=norm, cmap=self.__cmap)
            mapper.set_array(np.arange(min_val, max_val, (max_val - min_val) * 1. / self.__cmap_parts))

            # Plot regions
            for chrn, sp, ep, val in self.__bed_data:
                y = sp
                mid = (sp+ep)/2.
                dist = 0

                # if y locate at two ends or near centromeres, adjust width
                if mid <= 0.35 * ratio:
                    dist = (0.35 * ratio - mid) / ratio
                elif mid >= self.__chr_len_db[chrn] - 0.35 * ratio:
                    dist = (mid - (self.__chr_len_db[chrn] - 0.35 * ratio)) / ratio

                if self.__centro_db and chrn in self.__centro_db:
                    if self.__centro_db[chrn] - 0.35 * ratio <= sp <= self.__centro_db[chrn]:
                        dist = (sp - self.__centro_db[chrn] + 0.35 * ratio) / ratio
                    elif self.__centro_db[chrn] <= ep <= self.__centro_db[chrn] + 0.35 * ratio:
                        dist = (self.__centro_db[chrn] + 0.35 * ratio - ep) / ratio

                w = np.sqrt(0.1225 - dist ** 2) * 2
                x = chr_idx_db[chrn] - w / 2.
                h = ep - sp + 1
                if self.__orientation == 'horizontal':
                    x, y = y, x
                    w, h = h, w
                color = mapper.to_rgba(val)
                ax.add_patch(
                    plt.Rectangle((x, y), w, h, facecolor=color, edgecolor='none'))
            clb = plt.colorbar(mapper, shrink=0.5)
        elif self.__value_type == 'color':
            for chrn, sp, ep, color in self.__bed_data:
                y = sp
                mid = (sp + ep) / 2.
                dist = 0

                # if y locate at two ends or near centromeres, adjust width
                if mid <= 0.35 * ratio:
                    dist = (0.35 * ratio - mid) / ratio
                elif mid >= self.__chr_len_db[chrn] - 0.35 * ratio:
                    dist = (mid - (self.__chr_len_db[chrn] - 0.35 * ratio)) / ratio

                if self.__centro_db and chrn in self.__centro_db:
                    if self.__centro_db[chrn] - 0.35 * ratio <= sp <= self.__centro_db[chrn]:
                        dist = (sp - self.__centro_db[chrn] + 0.35 * ratio) / ratio
                    elif self.__centro_db[chrn] <= ep <= self.__centro_db[chrn] + 0.35 * ratio:
                        dist = (self.__centro_db[chrn] + 0.35 * ratio - ep) / ratio

                w = np.sqrt(0.1225 - dist ** 2) * 2

                x = chr_idx_db[chrn] - w / 2.
                h = ep - sp + 1
                if self.__orientation == 'horizontal':
                    x, y = y, x
                    w, h = h, w
                ax.add_patch(
                    plt.Rectangle((x, y), w, h, facecolor=color, edgecolor='none'))
        elif self.__value_type == 'marker':
            for chrn, sp, ep, marker, color in self.__bed_data:
                x = chr_idx_db[chrn]
                y = sp
                if self.__orientation == 'horizontal':
                    x, y = y, x
                if self.__s:
                    plt.scatter(x, y, s=self.__s, color=color, marker=marker)
                else:
                    plt.scatter(x, y, color=color, marker=marker)
        return clb


def chromosome(chr_len_db: dict,
               chr_order: list = None,
               bed_data: list = None,
               centro_db: dict = None,
               value_type: str = "numeric",
               orientation: str = "vertical",
               cmap: str = "gist_rainbow",
               cmap_parts: int = 100,
               s: any = None,
               **kwargs):
    plotter = _Chromosome(chr_len_db, chr_order, bed_data, centro_db, value_type, orientation, cmap, cmap_parts, s)

    if not plt:
        plt.figure()

    fig = plt.gcf()
    ax = plt.gca()
    clb = plotter.plot(ax, fig, kwargs)

    return fig, ax, clb



import matplotlib.pyplot as plt
import numpy as np
import pandas as pd


class _Manhattan(object):
    def __init__(self, data, threshold=0, color=None, threshold_line_color='blue', threshold_line_width=1,
                 block_line_width=1, log_base=0, reverse=False, xtick_labels=True, ytick_labels=True):
        if color is None:
            color = ['orange', 'green']
        if log_base < 0:
            raise ValueError("log base must be greater than or equal to 0")
        x_ticks = []
        x_max = 0
        tmp_y = []
        if not color:
            color = ['orange', 'green']
        '''
        Convert __data to pd.DataFrame
        '''
        if isinstance(data, dict):
            columns = sorted(data)
            data = pd.DataFrame(data, columns=columns)
            for col in columns:
                data[col][0] = np.add(data[col][0], x_max)
                x_ticks.append((max(data[col][0]) - x_max) / 2 + x_max)
                x_max = max(data[col][0])
                if log_base != 0:
                    data[col][1] = np.log(data[col][1]) / np.log(log_base)
                if reverse:
                    data[col][1] = -data[col][1]
                tmp_y.extend(data[col][1])
        elif isinstance(data, list) or isinstance(data, np.ndarray):
            columns = []
            data = pd.DataFrame(data)
            x_max = max(data[0])
            if log_base != 0:
                data[1] = np.log(data[1]) / np.log(log_base)
            if reverse:
                data[1] = -data[1]
            tmp_y.extend(data[1])
        else:
            raise ValueError("Unsupported data type")

        if not isinstance(threshold, list):
            threshold = np.array([threshold])
        else:
            threshold = np.array(threshold)

        if log_base != 0:
            threshold = np.log(threshold) / np.log(log_base)
        if reverse:
            threshold = -threshold

        if not isinstance(threshold_line_color, list):
            threshold_line_color = [threshold_line_color]

        self.__data = data
        self.__xlim = (0, x_max)
        self.__xmax = x_max
        y_min = min(tmp_y)
        y_max = max(tmp_y)
        self.__ymin = y_min
        self.__ymax = y_max
        if y_max < 0:
            y_max = 0
        if y_min > 0:
            y_min = 0
        self.__ylim = (y_min, y_max)
        self.__x_ticks = x_ticks
        self.__x_labels = columns
        self.__color = color
        self.__threshold = threshold
        self.__threshold_line_color = threshold_line_color
        self.__threshold_line_width = threshold_line_width
        self.__block_line_width = block_line_width
        self.__xtick_labels = xtick_labels
        self.__ytick_labels = ytick_labels

    def plot(self, ax, marker, s, kws):
        data = self.__data
        ax.set(xlim=self.__xlim)

        if len(self.__ylim) != 0:
            ax.set(ylim=self.__ylim)
        if self.__xtick_labels:
            if self.__x_ticks:
                ax.set(xticks=self.__x_ticks)
                ax.set_xticklabels(self.__x_labels)
        else:
            ax.set_xticklabels([])

        if not self.__ytick_labels:
            ax.set_yticklabels([])

        # Plot scatter of values
        if not self.__x_labels:
            if isinstance(self.__color, list):
                color = self.__color[0]
            else:
                color = self.__color
            ax.scatter(data[0], data[1], color=color, marker=marker, s=s, **kws)
        else:
            if isinstance(self.__color, list):
                color_cnt = len(self.__color)
                for i in range(0, len(self.__x_labels)):
                    col = self.__x_labels[i]
                    color_idx = i % color_cnt
                    ax.scatter(data[col][0], data[col][1], color=self.__color[color_idx], marker=marker, s=s, **kws)
                    if color_cnt == 1:
                        x = max(data[col][0])
                        ax.plot([x, x], [self.__ymin, self.__ymax], color='lightgrey', lw=self.__block_line_width,
                                linestyle=':')
            else:
                for col in self.__x_labels:
                    ax.scatter(data[col][0], data[col][1], color=self.__color, **kws)
                    x = max(data[col][0])
                    ax.plot([x, x], [self.__ymin, self.__ymax], color='lightgrey', lw=self.__block_line_width,
                            linestyle=':')

        ax.plot([0, self.__xmax], [0, 0], color='lightgrey', lw=self.__block_line_width)
        # Plot thresholds
        if not (len(self.__threshold) == 1 and self.__threshold[0] == 0):
            color_cnt = len(self.__threshold_line_color)
            for i in range(0, len(self.__threshold)):
                th = self.__threshold[i]
                color_idx = i % color_cnt
                ax.plot([0, self.__xmax], [th, th], color=self.__threshold_line_color[color_idx],
                        lw=self.__threshold_line_width, linestyle=':')

        ax.tick_params(axis='both', which='both', length=0)
        ax.spines['top'].set_linewidth(0.5)
        ax.spines['top'].set_color('lightgrey')
        ax.spines['bottom'].set_linewidth(0.5)
        ax.spines['bottom'].set_color('lightgrey')
        ax.spines['left'].set_linewidth(0.5)
        ax.spines['left'].set_color('lightgrey')
        ax.spines['right'].set_linewidth(0.5)
        ax.spines['right'].set_color('lightgrey')


def manhattan(data, threshold=0, color=None,
              threshold_line_color='blue', threshold_line_width=1,
              block_line_width=1, log_base=0, reverse=False,
              xtick_labels=True, ytick_labels=True, ax=None, marker='.',
              s=1, **kwargs):
    if color is None:
        color = ['orange', 'green']
    plotter = _Manhattan(data, threshold, color, threshold_line_color, threshold_line_width, block_line_width,
                         log_base, reverse, xtick_labels, ytick_labels)

    if not plt:
        plt.figure()
    fig = plt.gcf()
    ax = plt.gca()
    plotter.plot(ax, marker, s, kwargs)

    return fig, ax



import matplotlib.pyplot as plt


class _GeneCluster(object):

    def __init__(self,
                 gene_list: list,
                 edgecolor: any = None,
                 edgewidth: int = 1,
                 lw: int = 3):

        self.__gene_list = gene_list
        self.__edgecolor = edgecolor
        self.__edgewidth = edgewidth
        self.__lw = lw

    def plot(self, ax, fig):
        fig_w, fig_h = fig.get_size_inches() * fig.dpi
        ymax = fig_h * 10. / fig_w
        xticks = []
        xlabels = []
        min_pos = None
        max_pos = None
        for gn, sp, ep, _, _ in self.__gene_list:
            if not min_pos or sp < min_pos:
                min_pos = sp
            if not max_pos or ep > max_pos:
                max_pos = ep

            xticks.append((ep + sp) / 2.)
            xlabels.append(gn)
        for i in range(len(self.__gene_list)):
            gn, sp, ep, direct, color = self.__gene_list[i]
            if self.__edgecolor:
                if isinstance(self.__edgecolor, list):
                    edgecolor = self.__edgecolor[i]
                else:
                    edgecolor = self.__edgecolor
            else:
                edgecolor = None
            if direct == '+':
                ax.arrow(sp, 0, ep - sp + 1, 0, length_includes_head=True, head_length=min(max_pos / 50., ep - sp + 1),
                         width=0.4, head_width=0.75, facecolor=color, edgecolor=edgecolor,
                         zorder=99, lw=self.__edgewidth)
            else:
                ax.arrow(ep, 0, -(ep - sp + 1), 0, length_includes_head=True,
                         head_length=min(max_pos / 50., ep - sp + 1), width=0.4, head_width=0.75, facecolor=color,
                         edgecolor=edgecolor, zorder=99, lw=self.__edgewidth)
        ax.plot([min_pos, max_pos], [0, 0], color='lightgrey', lw=self.__lw, zorder=1)
        ax.set_ylim(-.5, max(.5, ymax - .5))
        ax.set_yticks([])
        ax.set_xticks(xticks, xlabels, rotation=-45, ha='center')
        for i in ax.spines:
            ax.spines[i].set_visible(False)
        ax.tick_params('both', length=0)


def genecluster(gene_list: list,
                edgecolor: any = None,
                edgewidth: int = 1,
                lw: int = 3
                ):
    plotter = _GeneCluster(gene_list, edgecolor, edgewidth, lw)

    if not plt:
        plt.figure()
    fig = plt.gcf()
    ax = plt.gca()
    plotter.plot(ax, fig)

    return fig, ax



from .chromosome import chromosome as chromosome
from .genecluster import genecluster as genecluster
from .manhattan import manhattan as manhattan
from .multialign import multialign as multialign


__version__ = "0.1.0.dev11"



from setuptools import setup

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='bioplotz',
    version='0.1.0.dev11',
    packages=['bioplotz'],
    url='https://github.com/sc-zhang/bioplotz',
    license='',
    author='Shengcheng Zhang',
    author_email='zsc-zhang@foxmail.com',
    description='A python package for drawing some bioinformatic pictures.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=['numpy',
                      'matplotlib',
                      'pandas']
)



import matplotlib.pyplot as plt
from matplotlib.transforms import Affine2D, offset_copy


class _MultiAlign(object):

    def __init__(self,
                 data: dict,
                 base_per_line: int = 80,
                 match_color: any = 'blue',
                 match_background_color: any = None,
                 mismatch_color: any = 'red',
                 mismatch_background_color: any = None,
                 highlight_positions: list = None,
                 highlight_color: any = 'green',
                 highlight_background_color: any = None):
        self.__data = data
        self.__base_per_line = base_per_line
        self.__match_color = match_color
        self.__match_background_color = match_background_color
        self.__mismatch_color = mismatch_color
        self.__mismatch_background_color = mismatch_background_color
        if highlight_positions:
            self.__highlight_positions = set(highlight_positions)
        else:
            self.__highlight_positions = None
        self.__highlight_color = highlight_color
        self.__highlight_background_color = highlight_background_color

    @staticmethod
    def __rainbow_text(x, y, strings, colors, background_colors, ax=None, **kwargs):
        """
        Take a list of *strings* and *colors* and place them next to each
        other, with text strings[i] being shown in colors[i].

        Parameters
        ----------
        x, y : float
            Text position in data coordinates.
        strings : list of str
            The strings to draw.
        colors : list of color
            The colors to use.
        ax : Axes, optional
            The Axes to draw into. If None, the current axes will be used.
        **kwargs
            All other keyword arguments are passed to plt.text(), so you can
            set the font size, family, etc.
        """
        if ax is None:
            ax = plt.gca()
        t = ax.transData
        fig = ax.figure
        canvas = fig.canvas

        for s, c, bc in zip(strings, colors, background_colors):
            if bc:
                text = ax.text(x, y, s, color=c,
                               bbox=dict(facecolor=bc, linewidth=0, pad=0),
                               transform=t, **kwargs)
            else:
                text = ax.text(x, y, s, color=c, transform=t, **kwargs)

            # Need to draw to update the text position.
            text.draw(canvas.get_renderer())
            ex = text.get_window_extent()
            # Convert window extent from pixels to inches
            # to avoid issues displaying at different dpi
            ex = fig.dpi_scale_trans.inverted().transform_bbox(ex)
            t = text.get_transform() + offset_copy(Affine2D(), fig=fig, x=ex.width, y=0)

    def plot(self, ax, **kwargs):
        aln_len = 0
        for gid in self.__data:
            aln_len = len(self.__data[gid])
            break
        seq_cnt = len(self.__data)
        seq_list = sorted(self.__data.keys())
        row_cnt = int(aln_len * 1. / self.__base_per_line)
        if row_cnt * self.__base_per_line < aln_len:
            row_cnt += 1
        total_row_cnt = (row_cnt + 2) * seq_cnt

        divide_repeat_cnt = int(self.__base_per_line / 5.)
        if divide_repeat_cnt * 5 < self.__base_per_line:
            divide_repeat_cnt += 1

        divide = '----+' * divide_repeat_cnt
        divide = divide[:self.__base_per_line]

        y_ticks = []
        y_labels = []
        for i in range(row_cnt):
            sp = i * self.__base_per_line
            ep = min(aln_len, sp + self.__base_per_line)

            divide_and_pos = "%s %d-%d" % (divide, sp + 1, ep)
            colors = ['black' for _ in range(len(divide_and_pos))]
            background_colors = ['white' for _ in range(len(divide_and_pos))]
            self.__rainbow_text(0, i * (seq_cnt + 2), list(divide_and_pos), colors, background_colors, **kwargs)
            colors = [self.__match_color for _ in range(ep - sp)]
            background_colors = [self.__match_background_color for _ in range(ep - sp)]

            for k in range(sp, ep):
                cur_set = set()
                for j in sorted(self.__data):
                    cur_set.add(self.__data[j][k])
                if len(cur_set) > 1:
                    colors[k - sp] = self.__mismatch_color
                    background_colors[k - sp] = self.__mismatch_background_color
                if self.__highlight_positions and k in self.__highlight_positions:
                    colors[k - sp] = self.__highlight_color
                    background_colors[k - sp] = self.__highlight_background_color
            for j in range(seq_cnt):
                gid = seq_list[j]
                y_ticks.append(i * (seq_cnt + 2) + j + 1)
                y_labels.append(gid)
                self.__rainbow_text(0, i * (seq_cnt + 2) + j + 1, self.__data[gid][sp: ep],
                                    colors, background_colors, **kwargs)
        ax.set_ylim(0, total_row_cnt)
        ax.set_xticks([])
        ax.invert_yaxis()
        ax.set_yticks(y_ticks, y_labels)
        for i in ax.spines:
            ax.spines[i].set_visible(False)
        ax.tick_params('both', length=0)


def multialign(data: dict,
               base_per_line: int = 80,
               match_color: any = 'blue',
               match_background_color: any = None,
               mismatch_color: any = 'red',
               mismatch_background_color: any = None,
               highlight_positions: list = None,
               highlight_color: any = 'green',
               highlight_background_color: any = None,
               **kwargs):
    plotter = _MultiAlign(data,
                          base_per_line,
                          match_color,
                          match_background_color,
                          mismatch_color,
                          mismatch_background_color,
                          highlight_positions,
                          highlight_color,
                          highlight_background_color)

    if not plt:
        plt.figure()
    fig = plt.gcf()
    ax = plt.gca()
    plotter.plot(ax, **kwargs)

    return fig, ax
